/* 
 * Packet Dumper-Listen on interface and dumps every packet
 * Copyright (c) 2006
 * Hardik Shah
 * http://www.secgeeks.com
 * All rights reserved.
 */

#include <stdlib.h>
#include <stdio.h>
#include<string.h>
#include <pcap.h>
#include"Packet_Header.h"


#define LINE_LEN 16

void UDPDecoder(const struct pcap_pkthdr *header, const u_char *pkt_data);
void TCPDecoder(const struct pcap_pkthdr *header, const u_char *pkt_data);
void print_payload(const u_char *payload, int len);
void YahooDecoder(char *payload);
void RediffDecoder(unsigned char *payload);
void AIMDecoder(unsigned char *payload);
void MSNDecoder(char *payload);

int size_payload;

main(int argc, char **argv)
{   
pcap_if_t *alldevs, *d;
u_int inum, i=0;
pcap_t *adhandle;
u_int netmask;
char packet_filter[] = "";
struct bpf_program fcode;
char errbuf[PCAP_ERRBUF_SIZE];
int res;
struct pcap_pkthdr *header;
u_char *pkt_data;
 struct tm *ltime;
 char timestr[16];
 pcap_dumper_t *dumpfile;
 ip_header *ih;// ip header declration
 u_int ip_len;//lendthg of IP Header


    printf("packetdump: prints the packets of the network using WinPcap.\n");
    printf("   Usage: pktdump_ex [-s source]\n\n"
           "   Examples:\n"
           "      pktdump_ex -s file://c:/temp/file.acp"
           "      pktdump_ex -s rpcap://\\Device\\NPF_{C8736017-F3C3-4373-94AC-9A34B7DAD998}\n\n");

    if(argc < 3)
    {

        printf("\nNo adapter selected: printing the device list:\n");
        /* The user didn't provide a packet source: Retrieve the local device list */
        if (pcap_findalldevs_ex(PCAP_SRC_IF_STRING, NULL, &alldevs, errbuf) == -1)
        {
            fprintf(stderr,"Error in pcap_findalldevs_ex: %s\n", errbuf);
            exit(1);
        }
        
        /* Print the list */
        for(d=alldevs; d; d=d->next)
        {
            printf("%d. %s\n    ", ++i, d->name);

            if (d->description)
                printf(" (%s)\n", d->description);
            else
                printf(" (No description available)\n");
        }
        
        if (i==0)
        {
            printf("\nNo interfaces found! Make sure WinPcap is installed.\n");
            return -1;
        }
        
        printf("Enter the interface number (1-%d):",i);
        scanf("%d", &inum);
        
        if (inum < 1 || inum > i)
        {
            printf("\nInterface number out of range.\n");

            /* Free the device list */
            pcap_freealldevs(alldevs);
            return -1;
        }
        
        /* Jump to the selected adapter */
        for (d=alldevs, i=0; i< inum-1 ;d=d->next, i++);
        
        /* Open the device */
        if ( (adhandle= pcap_open(d->name,
                            65536 /*snaplen*/,
                            PCAP_OPENFLAG_PROMISCUOUS /*flags*/,
                            20 /*read timeout*/,
                            NULL /* remote authentication */,
                            errbuf)
                            ) == NULL)
        {
            fprintf(stderr,"\nError opening adapter\n");
            return -1;
        }
    }
    else 
    {
        // Do not check for the switch type ('-s')
        if ( (adhandle= pcap_open(argv[2],
                            100 /*snaplen*/,
                            PCAP_OPENFLAG_PROMISCUOUS /*flags*/,
                            20 /*read timeout*/,
                            NULL /* remote authentication */,
                            errbuf)
                            ) == NULL)
        {
            fprintf(stderr,"\nError opening source: %s\n", errbuf);
            return -1;
        }
    }

/* Check the link layer. We support only Ethernet for simplicity. */
	if(pcap_datalink(adhandle) != DLT_EN10MB)
	{
		fprintf(stderr,"\nThis program works only on Ethernet networks.\n");
		/* Free the device list */
		pcap_freealldevs(alldevs);
		return -1;
	}
	else
		printf("we are in Ehernet.good.");
	
	if(d->addresses != NULL)
		/* Retrieve the mask of the first address of the interface */
		netmask=((struct sockaddr_in *)(d->addresses->netmask))->sin_addr.S_un.S_addr;
	else
		/* If the interface is without addresses we suppose to be in a C class network */
		netmask=0xffffff; 


	//compile the filter
	if (pcap_compile(adhandle, &fcode, packet_filter, 1, netmask) <0 )
	{
		fprintf(stderr,"\nUnable to compile the packet filter. Check the syntax.\n");
		/* Free the device list */
		pcap_freealldevs(alldevs);
		return -1;
	}
	
	//set the filter
	if (pcap_setfilter(adhandle, &fcode)<0)
	{
		fprintf(stderr,"\nError setting the filter.\n");
		/* Free the device list */
		pcap_freealldevs(alldevs);
		return -1;
	}
	
	/* Open the dump file */
	dumpfile = pcap_dump_open(adhandle, "dump.cap");

	if(dumpfile==NULL)
	{
		fprintf(stderr,"\nError opening output file\n");
		return -1;
	}
	printf("\nlistening on %s...\n", d->description);
	
    /* Read the packets */
    while((res = pcap_next_ex( adhandle, &header, (const unsigned char **) &pkt_data)) >= 0)
    {

        if(res == 0)
            /* Timeout elapsed */
            continue;
		/*comment out for raw packets

        // convert the timestamp to readable format 
    ltime=localtime(&header->ts.tv_sec);
    strftime( timestr, sizeof timestr, "%H:%M:%S", ltime);

    // print timestamp and length of the packet 
    printf("%s.%.6d len:%d ", timestr, header->ts.tv_usec, header->len);
        printf("Printing Raw Packet\n");
        // Print the packet 
        for (i=1; (i < header->caplen + 1 ) ; i++)
        {
            printf("%.2x ", pkt_data[i-1]);
            if ( (i % LINE_LEN) == 0) printf("\n");
        }
        
        printf("\n\n");     */

	/* save the packet on the dump file */
	pcap_dump(dumpfile, header, pkt_data);
		// retireve the position of the ip header 
       ih = (ip_header *) (pkt_data + 14); //length of ethernet header
	      ip_len = (ih->ver_ihl & 0xf) * 4;
		  //printf("protocol=%d\n",ih->proto);
	   switch(ih->proto)
	   {
		   case  IPPROTO_UDP:
			   printf("got an udp packet\n");
				UDPDecoder(header,pkt_data);
				break;

		   case IPPROTO_TCP :
			   printf("got an tcp packet\n");
			   TCPDecoder(header,pkt_data);
			   break;
	   }

	   res=NULL;
    }

	
    if(res == -1)
    {
        printf("Error reading the packets: %s\n", pcap_geterr(adhandle));
        return -1;
    }

    return 0;
}

void UDPDecoder(const struct pcap_pkthdr *header, const u_char *pkt_data)
{
	 ip_header *ih;// ip header declration
 u_int ip_len;//lendthg of IP Header
	udp_header *uh;//udp header declaration
    u_short sport,dport;//source port and destination port
	const char *payload;

	  
	/* retireve the position of the ip header */
       ih = (ip_header *) (pkt_data + 14); //length of ethernet header
	      ip_len = (ih->ver_ihl & 0xf) * 4;

	/*udp header position*/
    uh = (udp_header *) ((u_char*)ih + ip_len);

    /* convert from network byte order to host byte order */
    sport = ntohs( uh->sport );
    dport = ntohs( uh->dport );

    /* print ip addresses and udp ports */
    printf("%d.%d.%d.%d.%d -> %d.%d.%d.%d.%d\n",
        ih->saddr.byte1,
        ih->saddr.byte2,
        ih->saddr.byte3,
        ih->saddr.byte4,
        sport,
        ih->daddr.byte1,
        ih->daddr.byte2,
        ih->daddr.byte3,
        ih->daddr.byte4,
        dport);

      payload = (const char*)(pkt_data + 14 + sizeof(struct ip_header) + sizeof(struct udp_header));
          //payload[ntohs(uh->len)-8] = 0; 
	printf("payload:%s\n",payload);

}

void TCPDecoder(const struct pcap_pkthdr *header, const u_char *pkt_data)
{
	ip_header *ih;// ip header declration
    tcp_header *th;//tcp header declaration
    u_int ip_len;//lendthg of IP Header
	u_int tcp_len;
    u_short sport,dport;//source port and destination port
	const u_char *payload;
	
	  
	/* retireve the position of the ip header */
    ih = (ip_header *) (pkt_data +
        14); //length of ethernet header
	

    /* retireve the position of the tcp header */
    ip_len = (ih->ver_ihl & 0xf) * 4;//convert hex to decimal
    th = (tcp_header *) ((u_char*)ih + ip_len);
	tcp_len = TH_OFF(th)*4;

    /* convert from network byte order to host byte order */
    sport = ntohs( th->th_sport );
    dport = ntohs( th->th_dport );
    payload = (u_char *)(pkt_data + 14 + ip_len + tcp_len);
	
	/* compute tcp payload (segment) size */
	size_payload = ntohs(ih->tlen) - (ip_len + tcp_len);
	if(size_payload>0){
	/*for yahoo mesnger*/
	if ((sport==5050)||(dport==5050))
	YahooDecoder(payload);

	/*for Rediff Bol*/
	if (((sport==1863)||(dport==1863))&&(payload[1]==0x00)&&(payload[2]==0x00)&&(payload[3]==0x00))
	RediffDecoder(payload);

	/*for msn mesnger*/
//	if (((sport==1863)||(dport==1863))&&((payload[1]!=0x00)&&(payload[2]!=0x00)&&(payload[3]!=0x00)))
//	MSNDecoder(payload);
	}

	/*for AIM Protocol*/
	if (((sport==5190)||(dport==5190)))
		AIMDecoder(payload);


	/* print tcp details */
    printf("Source Port:%d\n Destination Port%d\n ",
               sport,
               dport);

	/* deal with printing binary payload data */
	printf("   Payload (%d bytes):  \n", size_payload);
	print_payload(payload, size_payload);
	printf("\n\n");
   
}

/*
 * print data in rows of 16 bytes: offset   hex   ascii
 *
 * 00000   47 45 54 20 2f 20 48 54  54 50 2f 31 2e 31 0d 0a   GET / HTTP/1.1..
 */
void
print_hex_ascii_line(const u_char *payload, int len, int offset)
{

	int i;
	int gap;
	const u_char *ch;

	/* offset */
	printf("%05d   ", offset);
	
	/* hex */
	ch = payload;
	for(i = 0; i < len; i++) {
		printf("%02x ", *ch);
		ch++;
		/* print extra space after 8th byte for visual aid */
		if (i == 7)
			printf(" ");
	}
	/* print space to handle line less than 8 bytes */
	if (len < 8)
		printf(" ");
	
	/* fill hex gap with spaces if not full line */
	if (len < 16) {
		gap = 16 - len;
		for (i = 0; i < gap; i++) {
			printf("   ");
		}
	}
	printf("   ");
	
	/* ascii (if printable) */
	ch = payload;
	for(i = 0; i < len; i++) {
		if (isprint(*ch))
			printf("%c", *ch);
		else
			printf(".");
		ch++;
	}

	printf("\n");

return;
}
/*
 * print packet payload data (avoid printing binary data)
 */
void
print_payload(const u_char *payload, int len)
{

	int len_rem = len;
	int line_width = 16;			/* number of bytes per line */
	int line_len;
	int offset = 0;					/* zero-based offset counter */
	const u_char *ch = payload;

	if (len <= 0)
		return;

	/* data fits on one line */
	if (len <= line_width) {
		print_hex_ascii_line(ch, len, offset);
		return;
	}

	/* data spans multiple lines */
	for ( ;; ) {
		/* compute current line length */
		line_len = line_width % len_rem;
		/* print line */
		print_hex_ascii_line(ch, line_len, offset);
		/* compute total remaining */
		len_rem = len_rem - line_len;
		/* shift pointer to remaining bytes to print */
		ch = ch + line_len;
		/* add offset */
		offset = offset + line_width;
		/* check if we have line width chars or less */
		if (len_rem <= line_width) {
			/* print last line and get out */
			print_hex_ascii_line(ch, len_rem, offset);
			break;
		}
	}

return;
}


void YahooDecoder(unsigned char *payload)
{

/* following is the yahoo packet structure
	struct YahooPacket{
		char Protocol[4];
		char Version[4];
		int Packlen;
		int Service;
		int Status;
		char Sessionid[4];
		char *data;
	}*YahooPacket;

	YahooPacket=payload;*/
int i,j;
int packlen;
char fBytes[5];
char *command=NULL;
char *key=NULL;
FILE *f1;
	
strcpy(fBytes,payload,4);
fBytes[5]='\0';
if (strcmp(fBytes,"YMSG")==0)
{
	printf("Application Protocol:%s",fBytes);
	
	/*print the version*/
if (payload[4]!=0) 
    printf("Protocol Version:%d\n",payload[4]);
else if (payload[5]!=0) 
    printf("Protocol Version:%d\n",payload[5]);
else if (payload[6]!=0)
    printf("Protocol Version:%d\n",payload[6]);
else if (payload[7]!=0)
    printf("Protocol Version:%d\n",payload[7]);


//here we will check if it is a msg or not if it is not a msg then we will simply discard this packet
if((payload[11]==0x06)&&(payload[13]=='U'))
{
//following code will determine the packet lenthg
packlen=((payload[8]&0xff)<<8)+payload[9]&0xff; //2 bytes==1 integer :)
i=19; //strip off yahoo header directly go for usefull data
j=0;
command=malloc((packlen - i) + 1);
command[0]='\0';
f1 = fopen ("yahoo.txt", "a+");
	while(i<packlen)
	{
	if((payload[i++]==0xc0)&&(payload[i]==0x80)) //c080 is the terminating string
		{
		printf("\n");
		//fprintf(f1,"\n");
		command[j]='\0';
		j=0;
		if(key!=NULL)
			free(key); //free up memory
		key=command;
		command=NULL;
		command=malloc(packlen - i + 1);
		command[0]='\0';
		}
	else
		if(payload[i]!=0xc0)
			{
			//printf("%c",payload[i]);
			if(strcmp(command,"")==0)
				if(key!=NULL)
				{
					if((strcmp(key,"1")!=0)&&(strcmp(key,"14")!=0)&&(strcmp(key,"5")!=0)&&(strcmp(key,"97")!=0)&&(strcmp(key,"63")!=0)&&(strcmp(key,";0")!=0)&&(strcmp(key,"64")!=0)&&(strcmp(key,"4")!=0)&&(strcmp(key,"252")!=0))
						fprintf(f1,"%s",key);
					else
						if(strcmp(key,"5")==0) //reciver so insert --> before it
							fprintf(f1,"-->");
						else
							if(strcmp(key,"14")==0) //message so insert a linefeed
								fprintf(f1,"\n");
							else
								fprintf(f1,"\n");  //end of message so insert a linefeed
					
				}

			command[j++]=payload[i];
							
				}
		}//end while

 fclose(f1);
}

}//if YMSG

}

void RediffDecoder(unsigned char *payload)
{
char *command;
char *SenderName;
char *ReciverName;
char *Message;
int i=0;
int j=0;
int k=0;
int curPos=0;
int Len;
FILE *f1;

 if((payload[1]==0x00)&&(payload[2]==0x00)&&(payload[3]==0x00))
 {
	 	command=malloc(payload[25]+1);
		curPos=29;
		while(i<payload[25])
		{
			curPos=29+i;
			command[i++]=payload[curPos];
		}
		command[i]='\0';
	
		if(strcmp("TextMessage",command)==0)
		{
		 curPos+=5;
		 Len=payload[curPos];
		 SenderName=malloc(Len+1);
		 curPos+=4;
		 j=curPos;
		 i=0;
			while(i<Len)
			{
				curPos=j+i;
				SenderName[i++]=payload[curPos];
				}
			SenderName[i]='\0';
			curPos++;
			Len=payload[curPos];
			ReciverName=malloc(Len+1);
			curPos+=4;
			j=curPos;
			i=0;
			
			while(i<Len)
			{
				curPos=j+i;
				ReciverName[i++]=payload[curPos];
				}
			ReciverName[i]='\0';
		//for message
		//by default it also contains the formating information like font,bol etc.
		//also we are ignoring the talkie(Audioables).
			curPos++;
			Len=payload[curPos];
			Message=malloc(Len+1);
			curPos+=4;
			j=curPos;
			i=36;
			
			while(i<Len)
			{
				curPos=j+i;
			if(payload[curPos]!='\0')
			Message[k++]=payload[curPos];
			i++;
				}
			Message[k]='\0';
		//end message

		f1 = fopen ("rediff.txt", "a+");
		fprintf(f1,"%s",SenderName);
//printf("SenderName:%s",SenderName);
		fprintf(f1,"-->");
		fprintf(f1,"%s",ReciverName);
//printf("ReciverName:%s\n",ReciverName);
		fprintf(f1,"\n");
		fprintf(f1,"%s",Message);
//printf("MEssage:%s",Message);
		fprintf(f1,"\n");
		fclose(f1);
		free(command);
		free(SenderName);
		free(ReciverName);
		free(Message);
		payload=NULL;
		}
	}


}

void AIMDecoder(unsigned char *payload)
{
char *ReciverName;
char *SenderName;
char *Message;
int Len;
int i;
int j;
int k;
int curPos;
int TLVLen;
int TLVSessionLen;
int TLVMsgLen;
int MsgLen;
int onlineLen;
int FeatureLen;
int BlockLen;
FILE *f1;

if(payload[0]==0x2a)
{
	printf("GOT AN AIM Packet");
	Len=((payload[4]&0xff)<<8)+payload[5]&0xff; //2 bytes==1 integer :)
	//printf("Packet Length:%d",Len);
	if((payload[8]==0x00)&&((payload[9]==0x07))) //this is an incoming msg
	{	ReciverName="Suspect";
		i=27;
		SenderName=malloc(payload[26]+1);
		j=0;
	while(j<payload[26])
	{
		curPos=i+j;
		SenderName[j++]=payload[curPos];
	}
	SenderName[j]='\0';
	//printf("SenderName:%s",SenderName);
	curPos+=7;
	TLVLen=((payload[curPos]&0xff)<<8)+payload[curPos+1]&0xff; //2 bytes==1 integer :)
	//printf("TLVLen:%d",TLVLen);
	curPos+=1;
	curPos+=TLVLen;
	curPos+=3;
	TLVSessionLen=((payload[curPos]&0xff)<<8)+payload[curPos+1]&0xff; //2 bytes==1 integer :)
	//printf("TLVSessionLen:%d",TLVSessionLen);
	curPos+=TLVSessionLen;
	curPos+=4;
	TLVMsgLen=((payload[curPos]&0xff)<<8)+payload[curPos+1]&0xff; //2 bytes==1 integer :)
	//printf("TLVMessageLen:%d",TLVMsgLen);
	curPos+=1;
	curPos+=TLVMsgLen;
	curPos+=3;
	onlineLen=((payload[curPos]&0xff)<<8)+payload[curPos+1]&0xff; //2 bytes==1 integer :)
	//printf("OnlineLen:%d",onlineLen);
	curPos+=1;
	curPos+=onlineLen;
	curPos+=3;
	MsgLen=((payload[curPos]&0xff)<<8)+payload[curPos+1]&0xff; //2 bytes==1 integer :)
	//printf("MessageLen:%d",MsgLen);
	curPos+=4;
	FeatureLen=((payload[curPos]&0xff)<<8)+payload[curPos+1]&0xff; //2 bytes==1 integer :)
	curPos+=1;
	curPos+=FeatureLen;
	curPos+=3;
	BlockLen=((payload[curPos]&0xff)<<8)+payload[curPos+1]&0xff; //2 bytes==1 integer :)
	//printf("%02x",payload[curPos]);
	//printf("%02x",payload[curPos+1]);
	//printf("BlockLen:%d",BlockLen);
	curPos+=6;
	i=0;
	Message=malloc(BlockLen-4+1);
	while(i<BlockLen-4)
	{
    	Message[i++]=payload[curPos+i];
			}

	Message[i]='\0';
	//printf("Message:%s\n",Message);

		f1 = fopen ("AIM.txt", "a+");
		fprintf(f1,"%s",SenderName);
//printf("SenderName:%s",SenderName);
		fprintf(f1,"-->");
		fprintf(f1,"%s",ReciverName);
//printf("ReciverName:%s\n",ReciverName);
		fprintf(f1,"\n");
		fprintf(f1,"%s",Message);
//printf("MEssage:%s",Message);
		fprintf(f1,"\n");
		fclose(f1);
		//free(SenderName);
		//free(ReciverName);
		//free(Message);
		payload=NULL;
	}
	else
		if((payload[8]==0x00)&&((payload[9]==0x06))) //this is an outgoing msg
		{
		  //printf("GOT AN OUTGOING MESSAGE");
		  SenderName="Suspect";
		i=27;
		ReciverName=malloc(payload[26]+1);
		j=0;
	while(j<payload[26])
	{
		curPos=i+j;
		ReciverName[j++]=payload[curPos];
	}
	ReciverName[j]='\0';
	//printf("ReciverName:%s",ReciverName);
	curPos+=3;
	TLVLen=((payload[curPos]&0xff)<<8)+payload[curPos+1]&0xff; //2 bytes==1 integer :)
	//printf("TLVLen:%d",TLVLen);
	curPos+=4;
	FeatureLen=((payload[curPos]&0xff)<<8)+payload[curPos+1]&0xff; //2 bytes==1 integer :)
	//printf("FeatureLen:%d",FeatureLen);
	curPos+=1;
	curPos+=FeatureLen;
	curPos+=3;
	BlockLen=((payload[curPos]&0xff)<<8)+payload[curPos+1]&0xff; //2 bytes==1 integer :)
	printf("BlockLen:%d",BlockLen);
	curPos+=6;
	i=0;
	Message=malloc(BlockLen-4+1);
	while(i<BlockLen-4)
	{
    	Message[i++]=payload[curPos+i];
			}

	Message[i]='\0';
	//printf("Message:%s\n",Message);
		f1 = fopen ("AIM.txt", "a+");
		fprintf(f1,"%s",SenderName);
	//printf("SenderName:%s",SenderName);
		fprintf(f1,"-->");
		fprintf(f1,"%s",ReciverName);
	//printf("ReciverName:%s\n",ReciverName);
		fprintf(f1,"\n");
		fprintf(f1,"%s",Message);
	//printf("MEssage:%s",Message);
		fprintf(f1,"\n");
		fclose(f1);
		//free(SenderName);
		//free(ReciverName);
		//free(Message);
		payload=NULL;



			}


	}

}

void MSNDecoder(unsigned char *payload)
{
char fBytes[4];
char *SenderName;
char *ReciverName;
char *Message;
char flg=0;
int i=0;
int j=0;
int k=0;
if (strlen(payload)>3){
strcpy(fBytes,payload,3);
fBytes[4]='\0';
if (strcmp(fBytes,"MSG")==0)
{
	printf("Application Protocol:MSN");

	if((0<=payload[4])&&(payload[4]<=9))
	{
		//0d0a0d0a
		while((i<size_payload)&&(flg=0))
		{
			if((payload[i]==0x0d)&&(payload[i]==0x0a)&&(payload[i]==0x0d)&&(payload[i]==0x0a))
			{	SenderName="Suspect";
				flg=1;
				Message=malloc(size_payload-i+1);
				for(j=i;j<size_payload;j++)
				{
					Message[k++]=payload[j];
				}
				Message[k]='\0';
			}

		}

	}
	
	}
}
}